﻿using System;

namespace SimpleSudoku
{
    class Program
    {
        static void Main(string[] args)
        {
            try
            {
                // Read from stdin
                Console.InputEncoding = System.Text.Encoding.UTF8;
                string[] input = Console.In.ReadToEnd().Split(new char[] { ' ', '\n', '\r' }, StringSplitOptions.RemoveEmptyEntries);
                
                // Convert to int array, change numbers to 0-8 instead of 1-9.
                int[] s = new int[81];
                for (int i = 0; i < 81; i++) s[i] = Convert.ToInt32(input[i]);
               
                // Solve the sudoku.
                if (Solve(ref s, 0, s[0]))
                    PrintTable(s);
                else
                    Console.WriteLine("No solution found.");
               
            }
            catch(Exception ex)
            {
                Console.Error.WriteLine("Error! Something went wrong.");
                Console.Error.WriteLine(ex.ToString());
            }
        }

        // Recursive function to solve a Sudoku.
        static bool Solve(ref int[] s,int p, int v)
        {
            s[p] = v;                                   // Enter value v in cell p.
            while (p < 81 && s[p] > 0) p++;             // Move forward to an empty cell.
            if (p == 81) return true;                   // If no more cells, we are finished.

            bool[] n = new bool[10];                    // Lets calculate allowed numbers in the cell.
            for (int i = 0; i < 9; i++)                 // For each of the 9 neighbouring cells,
                n[s[p/27*27 + i/3*9 + p%9/3*3 + i%3]] = // in the same box as p,
                n[s[p/9*9 + i]] =                       // in the same row as p,     
                n[s[p%9 + 9*i]] = true;                 // in the same col as p; disallow the number. 
          
            for (int i = 1; i <= 9; i++)                // Now we try the allowed numbers. 
                if (!n[i] && Solve(ref s, p, i))        // If number is allowed, try solving again.
                    return true;                        // Return success if solved.;
                
            s[p] = 0;                                   // If all allowed colours are tested,
            return false;                               // empty the cell and go back.
        }

        // Print a sudoku table to the console window.
        static void PrintTable(int[] s)
        {
            for (int i = 0; i < 81; i++)
                Console.Write(s[i] + " " + (i % 3 == 2 ? " " : "") + (i % 9 == 8 ? "\n" : "") + (i % 27 == 26 ? "\n" : ""));
            Console.WriteLine();
        }
    }
}
